/******************************************************************************/
/*                                                                            */
/*                                  bic.h                                     */
/*                                                                            */
/*                        Audio Segmentation Library                          */
/*                                                                            */
/* Guig                                                             July 2003 */
/* -------------------------------------------------------------------------- */
/*  Copyright (C) 2003 IRISA                                                  */
/*                                                                            */
/*  This program is free software; you can redistribute it and/or             */
/*  modify it under the terms of the GNU General Public License               */
/*  as published by the Free Software Foundation; either version 2            */
/*  of the License, or (at your option) any later version.                    */
/*                                                                            */
/*  This program is distributed in the hope that it will be useful,           */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of            */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             */
/*  GNU General Public License for more details.                              */
/*                                                                            */
/*  You should have received a copy of the GNU General Public License         */
/*  along with this program; if not, write to the Free Software               */
/*  Foundation, Inc., 59 Temple Place - Suite 330,                            */
/*  Boston, MA  02111-1307, USA.                                              */
/*                                                                            */
/******************************************************************************/
/*
 * CVS log:
 *
 *   $Author: ggravier $
 *   $Date: 2003/08/22 14:15:10 $
 *   $Revision: 1.5 $
 *
 */

/*
 * BIC segmentation utilities.
 */

#ifndef _bic_h_
# define _bic_h_

# include <audioseg.h>

# define BIC_OUTPUT_STRING "unk"

     /* ---------------------------- */
     /* ----- type definitions ----- */
     /* ---------------------------- */
/*
 * BIC segmentation window - the window is stored in the buffer which
 * is filled from stream f whenever necessary. The frames in the
 * buffer start at (absolute) index is and the number of frames is
 * given by the buffer size. The offset is used to get a buffer frame
 * index given an absolute frame index.
 */
typedef struct {
  spfstream_t *f;                /* stream to segment                        */
  unsigned long maxi;             /* maximum frame index in stream            */
  spfbuf_t *buf;                  /* window feature buffer                    */
  unsigned long is;               /* absolute window start index              */
  unsigned long im;               /* absolute window middle index             */
  unsigned long ie;               /* asbolute window end index                */
  unsigned long offset;           /* offset between abs. and buffer indexes   */
  double cp;                      /* complexity penalty                       */
} bicwin_t;


     /* ------------------------------- */
     /* ----- function prototypes ----- */
     /* ------------------------------- */

/*
 * Initialize a BIC window for a given stream.
 */
bicwin_t *bic_init(
  spfstream_t *,                  /* input feature stream                     */
  unsigned long,                  /* start frame index                        */
  unsigned long,                  /* original window size                     */
  unsigned long,                  /* maximum frame index in input stream      */
  int                             /* use full covariance or not?              */
);

/*
 * Fill in window buffer until the required number of frames is read
 * or until the end of stream. Return the actual frame length or 0 in
 * case of error.
 */
unsigned long bic_win_fill(
  bicwin_t *,                     /* pointer to BIC window                    */
  unsigned long,                  /* start frame index                        */
  unsigned long                   /* required number of frames                */
);

/*
 * Free memory allocated by BIC init.
 */
void bic_reset(
  bicwin_t *                      /* pointer to BIC window                    */
);

/*
 * Search for a change point in a window. If a change is found, set
 * the frame index of the first frame after the change and return
 * 1. Return 0 otherwise.
 */
int bic_change_search(
  bicwin_t *,                     /* pointer to BIC window                    */
  unsigned long,                  /* increment in frames                      */
  float,                          /* weight of the complexity penalty         */
  unsigned long *                 /* change point index                       */
);

/* 
 * Compute delta BIC distance for a window.
 */
double dbic(
  bicwin_t *,                     /* pointer to BIC window                    */
  float                           /* weight of the complexity penalty         */
);

# ifdef _bic_c_

#  define BIC_DIAG_COV 0
#  define BIC_FULL_COV 1

static int _ctype = BIC_DIAG_COV;

#  ifdef HAVE_GSL

#  include <gsl/gsl_blas.h>
#  include <gsl/gsl_eigen.h>

static gsl_matrix *_S = NULL;
static gsl_vector *_m = NULL;
static gsl_eigen_symm_workspace *_ws = NULL;
static gsl_vector *_ev = NULL;

#  endif /* HAVE_GSL */

static double *_dm = NULL;
static double *_dS = NULL;

# endif /* _bic_c_ */

#endif /* _bic_h_ */
