/******************************************************************************/
/*                                                                            */
/*                                  hmm.h                                     */
/*                                                                            */
/*                        Audio Segmentation Library                          */
/*                                                                            */
/* Mike                                                              May 2003 */
/* -------------------------------------------------------------------------- */
/*  Copyright (C) 2002 IRISA                                                  */
/*                                                                            */
/*  This program is free software; you can redistribute it and/or             */
/*  modify it under the terms of the GNU General Public License               */
/*  as published by the Free Software Foundation; either version 2            */
/*  of the License, or (at your option) any later version.                    */
/*                                                                            */
/*  This program is distributed in the hope that it will be useful,           */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of            */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             */
/*  GNU General Public License for more details.                              */
/*                                                                            */
/*  You should have received a copy of the GNU General Public License         */
/*  along with this program; if not, write to the Free Software               */
/*  Foundation, Inc., 59 Temple Place - Suite 330,                            */
/*  Boston, MA  02111-1307, USA.                                              */
/*                                                                            */
/******************************************************************************/
/*
 * CVS log:
 *
 *   $Author: ggravier $
 *   $Date: 2003/08/25 13:32:45 $
 *   $Revision: 1.2 $
 *
 */

/*
 * HMM utilities.
 */

#ifndef _hmm_h_
# define _hmm_h_

# include <audioseg.h>

# define HMM_NULL_STATE -1

     /* ---------------------------- */
     /* ----- type definitions ----- */
     /* ---------------------------- */
typedef struct {
  float *w;                          /* component weights                         */
  char *name;                        /* state name                                */
  gmm_t *pdf;                        /* state pdf                                 */
  float complexity;                  /* pdf complexity                            */ 
} state_t; 

typedef struct {
  unsigned short nstates;            /* number of states                          */
  state_t *state;                    /* state array                               */ 
  float *entryp;                     /* entry probabilities                       */
  float **transp;                    /* transition probabilities                  */
  float *exitp;                      /* exit probabilities                        */ 
} hmm_t;

     /* ------------------------------- */
     /* ----- function prototypes ----- */
     /* ------------------------------- */

/* allocate memory for a HMM */
hmm_t *hmm_alloc(
  unsigned short                     /* number of states                           */
);

/* free memory allocated for a HMM */
void hmm_free(
  hmm_t *                            /* HMM address                                */
);  

/* read a HMM from file */
hmm_t *hmm_read(
  const char *                       /* input filename                             */
);

# ifdef _hmm_c_
#  define HMM_COMMENT_CHAR '#'
#  define HMM_MAX_LINE_LENGTH 1024
#  define NUMBER_OF_STATES 1
#  define STATE_DEFINITION 2
#  define STATE_ENTRY_PROB 3
#  define STATE_TRANS_PROB 4
#  define STATE_EXIT_PROB 5
#  define HMM_END 6
# endif /* _hmm_c_ */

#endif /* _hmm_h_ */
