/******************************************************************************/
/*                                                                            */
/*                                   vq.c                                     */
/*                                                                            */
/*                        Audio Segmentation Library                          */
/*                                                                            */
/* Guig                                                         November 2002 */
/* -------------------------------------------------------------------------- */
/*  Copyright (C) 2002 IRISA                                                  */
/*                                                                            */
/*  This program is free software; you can redistribute it and/or             */
/*  modify it under the terms of the GNU General Public License               */
/*  as published by the Free Software Foundation; either version 2            */
/*  of the License, or (at your option) any later version.                    */
/*                                                                            */
/*  This program is distributed in the hope that it will be useful,           */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of            */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             */
/*  GNU General Public License for more details.                              */
/*                                                                            */
/*  You should have received a copy of the GNU General Public License         */
/*  along with this program; if not, write to the Free Software               */
/*  Foundation, Inc., 59 Temple Place - Suite 330,                            */
/*  Boston, MA  02111-1307, USA.                                              */
/*                                                                            */
/******************************************************************************/
/*
 * CVS log:
 *
 *   $Author: ggravier $
 *   $Date: 2003/11/20 15:50:58 $
 *   $Revision: 1.4 $
 *
 */

#define _vq_c_

#include <vq.h>

/* ----------------------------------------------------------------- */
/* ----- vqacc_t *vq_acc_alloc(unsigned short, unsigned short) ----- */
/* ----------------------------------------------------------------- */
/*
 * Allocate memory for the accumulators.
 */
vqacc_t *vq_acc_alloc(unsigned short n, unsigned short d)
{
  unsigned short i;
  vqacc_t *p;

  if ((p = (vqacc_t *)malloc(n * sizeof(vqacc_t))) == NULL)
    return(NULL);

  for (i = 0; i < n; i++)
    p[i].m = p[i].v = NULL;

  for (i = 0; i < n; i++) {
    p[i].m = (double *)malloc(d * sizeof(double));
    p[i].v = (double *)malloc(d * sizeof(double));
    
    if (p[i].m == NULL || p[i].v == NULL) {
      vq_acc_free(p, n);
      return(NULL);
    }
  }

  return(p);
}

/* ------------------------------------------------------- */
/* ----- void vq_acc_free(vqacc_t *, unsigned short) ----- */
/* ------------------------------------------------------- */
/*
 * Free allocated memory.
 */
void vq_acc_free(vqacc_t *p, unsigned short n)
{
  unsigned short i;

  if (p) {
    for (i = 0; i < n; i++) {
      if (p[i].m)
	free(p[i].m);
      if (p[i].v)
	free(p[i].v);
    }
  }
  free(p);
}

/* ------------------------------------------------------------------------ */
/* ----- void vq_acc_reset(vqacc_t *, unsigned short, unsigned short) ----- */
/* ------------------------------------------------------------------------ */
/*
 * Zero accumulators.
 */
void vq_acc_reset(vqacc_t *p, unsigned short n, unsigned short d)
{
  unsigned short i, j;

  for (i = 0; i < n; i++)
    for (j = 0; j < d; j++) {
      p[i].m[j] = 0.0;
      p[i].v[j] = 0.0;
    }
}

/* ------------------------------------------------------------- */
/* ----- int update(vqtree_t *, vqacc_t *, unsigned short) ----- */
/* ------------------------------------------------------------- */
/*
 * Update leaf means and variances from accumulators
 */
int update(vqtree_t *tree, vqacc_t *acc, unsigned short offset)
{
  unsigned short i, j;
  double x, nn, *m, *v;
  vqacc_t *p;

  for (i = 0; i < tree->nleaves; i++) {
    if (tree->node[offset+i].npts == 0) {
      fprintf(stderr, "sgminit update() -- empty cluster %d\n", offset+i);
      return(1);
    }

    p = acc + i;
    nn = 1.0 / (double)(tree->node[offset+i].npts);

    m = tree->node[offset+i].m;
    v = tree->node[offset+i].v;

    for (j = 0; j < tree->dim; j++) {
      x = *(p->m + j) * nn;

      *(m+j) = x;
      *(v+j) = *(p->v + j) * nn - x * x;
    }
  }

  return(0);
}

/* ---------------------------------------------------------------------------------------------- */
/* ----- unsigned short quantize(spf_t *, double *, unsigned short, vqnode_t *, vqnode_t *) ----- */
/* ---------------------------------------------------------------------------------------------- */
/*
 * Quantize a single vector between the left and right nodes. 
 */
unsigned short quantize(spf_t *yt, double *w, unsigned short dim, vqnode_t *left, vqnode_t *right, double *d)
{
  unsigned short j;
  double d1, d2, x, v, *mp1, *mp2;
  
  mp1 = left->m;
  mp2 = right->m;
  
  d1 = d2 = 0.0;

  if (w)
    for (j = 0; j < dim; j++) {
      x = (double)*(yt+j);
      v = x - *(mp1+j);
      d1 += v * v * *(w+j);
      v =  x - *(mp2+j);
      d2 += v * v * *(w+j);
    }
  else
    for (j = 0; j < dim; j++) {
      x = (double)*(yt+j);
      v = x - *(mp1+j);
      d1 += v * v;
      v =  x - *(mp2+j);
      d2 += v * v;
    }

  if (d)
    *d = (d1 < d2) ? d1 : d2;
  
  return((d1 < d2) ? 0 : 1);
}

/* -------------------------------------------------- */
/* ----- void split(vqtree_t *, unsigned short) ----- */
/* -------------------------------------------------- */
/*
 * Split nodes.
 * WARNING : 
 */
void split(vqtree_t *tree)
{
  unsigned short left, right, i, j, nnodes;
  double a, *v, *m, *lm, *rm, x;

  nnodes = tree->nnodes;
  for (i = 0; i < nnodes; i++)
    if (is_leaf(tree->node + i)) {
      left = tree->nnodes;
      right = left + 1;
      
      tree->node[i].child[0] = left;
      tree->node[left].parent = i;
      tree->node[i].child[1] = right;
      tree->node[right].parent = i;

      /* 
	 WATCH OUT: here's an ugly hack!!!! We cheat here to make
	 believe that the childs of a newly created leaf are the leaf
	 itself and its brother. This makes life a lot easier in
	 partition() since we don't have to worry whether we partition
	 accross the children or accross the brothers (yeah, that's a
	 family story!). Don't forget to fix that at some point or
	 you'll end out with a corrupted tree structure! You've been
	 warned ;) .  
      */
      tree->node[left].child[0] = left;
      tree->node[left].child[1] = right;
      tree->node[right].child[0] = right;
      tree->node[right].child[1] = left;

      m = tree->node[i].m;
      v = tree->node[i].v;
      lm = tree->node[left].m;
      rm = tree->node[right].m;
      
      for (j = 0; j < tree->dim; j++) {
	a = SPLIT_VAR_FRAC * *(v+j);
	x = *(m+j);
	*(lm+j) = x + a;
	*(rm+j) = x - a;
      }
      
      tree->nnodes += 2;
      tree->nleaves += 1;
    }
}

/* -------------------------------------------------------------------------------------------------- */
/* ----- vqtree_t *init_tree(unsigned short, unsigned short, double *, double *, unsigned long) ----- */
/* -------------------------------------------------------------------------------------------------- */
/*
 * Initialize the tree root using the global mean and variance.
 *
 * Since the tree is binary, there must be 2n-1 nodes to have n leaves.
 */
vqtree_t *init_tree(unsigned short nleaves, unsigned short dim, double *m, double *v, unsigned long nframes)
{
  vqtree_t *p;
  vqnode_t *node;
  unsigned short i, j, nnodes;
  double *mp, *vp;

  /* allocate and initialize the global tree structure */
  if ((p = (vqtree_t *)malloc(sizeof(vqtree_t))) == NULL)
    return(NULL);
  nnodes = 2 * nleaves - 1;
  if ((p->node = (vqnode_t *)malloc(nnodes * sizeof(vqnode_t))) == NULL) {
    free(p);
    return(NULL);
  }
  
  p->dim = dim;
  p->nanodes = nnodes;

  for (i = 0; i < nnodes; i++) {
    node = p->node + i;
    node->npts = 0;
    node->m = node->v = NULL;
    node->parent = node->child[0] = node->child[1] = VQ_NULL_ID;
  }

  for (i = 0; i < nnodes; i++) {    
    node = p->node + i;
    node->m = (double *)malloc(dim * sizeof(double));
    node->v = (double *)malloc(dim * sizeof(double));
    if (node->m == NULL || node->v == NULL) {
      free_tree(p);
      return(NULL);
    }
  }

  /* initialize the tree to the root with the global data means and variance */
  p->nnodes = 1;
  p->nleaves = 1;
  p->node->npts = nframes;

  mp = p->node->m;
  vp = p->node->v;

  for (j = 0; j < dim; j++) {
    *(mp+j) = *(m+j);
    *(vp+j) = *(v+j);
  }

  return(p);
}

/* -------------------------------------- */
/* ----- void free_tree(vqtree_t *) ----- */
/* -------------------------------------- */
/*
 * Free memory allocated to a tree.
 */
void free_tree(vqtree_t *p)
{
  unsigned long i;

  if (p) {
    if (p->node) {
      for (i = 0; i < p->nanodes; i++) {
	if (p->node[i].m)
	  free(p->node[i].m);
	if (p->node[i].v)
	  free(p->node[i].v);
      }
      free(p->node);
    }
    free(p);
  }
}

/* ----------------------------------------------- */
/* ----- void print_tree(FILE *, vqtree_t *) ----- */
/* ----------------------------------------------- */
void print_tree(FILE *f, vqtree_t *p)
{
  unsigned short i, j;
  vqnode_t *node;

  fprintf(f, "%u %u %u\n", p->nnodes, p->nleaves, p->dim);

  for (i = 0; i < p->nnodes; i++) {
    node = p->node + i;
    fprintf(f, "%u %ld %ld %ld %lu\n", i, node->parent, node->child[0], node->child[1], node->npts);
    for (j = 0; j < p->dim; j++)
      fprintf(f, j ? " %-10.4f" : " %-10.4f", node->m[j]);
    fprintf(f, "\n");
    for (j = 0; j < p->dim; j++)
      fprintf(f, j ? " %-10.4f" : " %-10.4f", node->v[j]);
    fprintf(f, "\n");
  }
}

#undef _vq_c_

