/******************************************************************************/
/*                                                                            */
/*                                   vq.h                                     */
/*                                                                            */
/*                        Audio Segmentation Library                          */
/*                                                                            */
/* Guig                                                         November 2002 */
/* -------------------------------------------------------------------------- */
/*  Copyright (C) 2002 IRISA                                                  */
/*                                                                            */
/*  This program is free software; you can redistribute it and/or             */
/*  modify it under the terms of the GNU General Public License               */
/*  as published by the Free Software Foundation; either version 2            */
/*  of the License, or (at your option) any later version.                    */
/*                                                                            */
/*  This program is distributed in the hope that it will be useful,           */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of            */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             */
/*  GNU General Public License for more details.                              */
/*                                                                            */
/*  You should have received a copy of the GNU General Public License         */
/*  along with this program; if not, write to the Free Software               */
/*  Foundation, Inc., 59 Temple Place - Suite 330,                            */
/*  Boston, MA  02111-1307, USA.                                              */
/*                                                                            */
/******************************************************************************/
/*
 * CVS log:
 *
 *   $Author: ggravier $
 *   $Date: 2003/11/20 15:50:58 $
 *   $Revision: 1.3 $
 *
 */

/*
 * Vector quantization utilities
 */

#ifndef _vq_h_
# define _vq_h_

# include <script.h>

# define VQ_NULL_ID -1

     /* ---------------------------- */
     /* ----- type definitions ----- */
     /* ---------------------------- */

typedef struct {
  long parent;                     /* parent node ID                        */
  long child[2];                   /* child node IDs                        */
  unsigned long npts;              /* cluster population                    */
  double *m;                       /* cluster mean                          */
  double *v;                       /* cluster variance                      */
} vqnode_t;

typedef struct {
  unsigned short nanodes;          /* number of nodes allocated             */
  unsigned short nnodes;           /* number of actual nodes                */
  unsigned short nleaves;          /* number of leaves                      */
  unsigned short dim;              /* codeword dimension                    */
  vqnode_t *node;                  /* table of node definitions             */
} vqtree_t;

typedef struct {
  double *m;                       /* cluster mean                          */
  double *v;                       /* cluster variance                      */
} vqacc_t;


     /* ------------------------------- */
     /* ----- function prototypes ----- */
     /* ------------------------------- */

/*
 * Allocate memory for a tree and initialize the root.
 */
vqtree_t *init_tree(
  unsigned short,                  /* target number of leaves               */
  unsigned short,                  /* feature vector dimension              */
  double *,                        /* root cluster mean vector              */
  double *,                        /* root cluster variance vector          */
  unsigned long                    /* root cluster size                     */
);

/*
 * Free memory allocated to a tree.
 */
void free_tree(
  vqtree_t *                       /* tree address                          */
);

/*
 * Print a tree to stream.
 */
void print_tree(
  FILE *,                          /* output stream                         */
  vqtree_t *                       /* tree address                          */
);

/*
 * Allocate memory for accumulators
 */
vqacc_t *vq_acc_alloc(
  unsigned short,                  /* accumulator size                      */
  unsigned short                   /* feature vector dimension              */
);

/*
 * Zero accumulators.
 */
void vq_acc_reset(
  vqacc_t *,                       /* accumulators                          */
  unsigned short,                  /* accumulator size                      */
  unsigned short                   /* feature vector dimension              */
);

/*
 * Free memory allocated to the accumulators
 */
void vq_acc_free(
  vqacc_t *,                       /* accumulators                          */
  unsigned short                   /* accumulator size                      */
);

/*
 * Split each leaf into two new clusters.
 */
void split(
  vqtree_t *                       /* tree address                          */
);

/*
 * Update leaf clusters - offset is the offset between the node IDs and
 * the accumulators index, assuming that the leaves all have contiguous
 * IDs (which is actually the case by construction so far). Returns 0
 * if everything's ok or 1 if an empty cluster was found.
 */
int update(
  vqtree_t *,                      /* tree address                          */
  vqacc_t *,                       /* accumulators                          */
  unsigned short                   /* offset                                */
);

/*
 * Quantize a vector between two nodes: return 0 for the left node and
 * 1 for the right node.
 */
unsigned short quantize(
  spf_t *,                         /* feature vector                        */
  double *,                        /* weighting vector (mahalanobis)        */
  unsigned short,                  /* feature vector dimension              */
  vqnode_t *,                      /* left node                             */
  vqnode_t *,                      /* right node                            */
  double *                         /* returned distortion (or NULL)         */
);

# define is_leaf(p) (((p)->child[0] == VQ_NULL_ID) && ((p)->child[1] == VQ_NULL_ID))

# ifdef _vq_c_
#  define SPLIT_VAR_FRAC 0.1       /* variance fraction for split           */
# endif /* _vq_c_ */

#endif /* _vq_h_ */
