#!/bin/sh -x
#
# Test sgmestim program
#
# $Author: ggravier $
# $Date: 2003/08/22 14:15:12 $
# $Revision: 1.3 $
#


prog=sgmestim

echo "checking ${prog}:"

if test ! -x ../src/${prog}; then
  echo "  you should probably build ${prog} before running test..."
  exit 1
fi

mkdir -p log

#
# check usage 
#
../src/${prog} --help > log/${prog}.stdout 2> log/${prog}.stderr
diff log/${prog}.stdout ref/${prog}.usage > /dev/null 2> /dev/null
if test $? -eq 0 -a x`awk '{print NR}' log/${prog}.stderr | tail -1` = x; then
  echo "  usage test passed"
else
  echo "  usage test failed (see log/${prog}.stderr for details)"
  exit 1
fi
\rm -f log/*

#
# ML estimation
#
../src/${prog} -f 0.01 -i 2 -o log/know1.gmm ref/init.2 dat/know1.lfcc > log/${prog}.stdout 2> log/${prog}.stderr
diff log/know1.gmm ref/estim.1 > /dev/null 2> /dev/null
if test $? -eq 0 -a x`awk '{print NR}' log/${prog}.stderr | tail -1` = x; then
  echo "  ML estimation test passed"
else
  echo "  ML estimation test failed (see log/${prog}.stderr for details)"
  exit 1
fi
\rm -f log/*

#
# MAP estimation
#
../src/${prog} -f 0.01 -i 1 -m 10 -o log/know1.gmm ref/init.2 dat/know1.lfcc > log/${prog}.stdout 2> log/${prog}.stderr
diff log/know1.gmm  ref/estim.2 > /dev/null 2> /dev/null
if test $? -eq 0 -a x`awk '{print NR}' log/${prog}.stderr | tail -1` = x; then
  echo "  MAP estimation test passed"
else
  echo "  MAP estimation test failed (see log/${prog}.stderr for details)"
  exit 1
fi
\rm -f log/*

#
# check script support 
#
../src/${prog} -f 0.01 -i 2 -L dat/script.lst -o log/know1.gmm ref/init.2 > log/${prog}.stdout 2> log/${prog}.stderr
diff log/know1.gmm  ref/estim.3 > /dev/null 2> /dev/null
if test $? -eq 0 -a x`awk '{print NR}' log/${prog}.stderr | tail -1` = x; then
  echo "  script test passed"
else
  echo "  script test failed (see log/${prog}.stderr for details)"
  exit 1
fi
\rm -f log/*

#
# check segment chopping
#
../src/${prog} -f 0.01 -i 2 -L dat/script.lst -x spk1 -o log/know1.gmm ref/init.2 > log/${prog}.stdout 2> log/${prog}.stderr
diff log/know1.gmm  ref/estim.4 > /dev/null 2> /dev/null
if test $? -eq 0 -a x`awk '{print NR}' log/${prog}.stderr | tail -1` = x; then
  echo "  chopping test passed"
else
  echo "  chopping test failed (see log/${prog}.stderr for details)"
  exit 1
fi
\rm -f log/*

rmdir log