#!/bin/sh
#
# Test sgminit program
#
# $Author: ggravier $
# $Date: 2003/08/22 14:15:12 $
# $Revision: 1.3 $
#


prog=sgminit

echo "checking ${prog}:"

if test ! -x ../src/${prog}; then
  echo "  you should probably build ${prog} before running test..."
  exit 1
fi

mkdir -p log

#
# check usage 
#
../src/${prog} --help > log/${prog}.stdout 2> log/${prog}.stderr
diff log/${prog}.stdout ref/${prog}.usage > /dev/null 2> /dev/null
if test $? -eq 0 -a x`awk '{print NR}' log/${prog}.stderr | tail -1` = x; then
  echo "  usage test passed"
else
  echo "  usage test failed (see log/${prog}.stderr for details)"
  exit 1
fi
\rm -f log/*

#
# check basic initialization
#
../src/${prog} -n 8 -f 1.0 -c log/know1.var log/know1.gmm dat/know1.lfcc > log/${prog}.stdout 2> log/${prog}.stderr
diff log/know1.gmm ref/init.1 > /dev/null 2> /dev/null
if test $? -eq 0 -a x`awk '{print NR}' log/${prog}.stderr | tail -1` = x; then
  diff log/know1.var ref/init.var > /dev/null 2> /dev/null
  if [ $? -eq 0 ]; then
    echo "  initialization test passed"
  else
    echo "  initialization test failed (see log/${prog}.stderr for details)"
    exit 1
  fi
else
  echo "  initialization test failed (see log/${prog}.stderr for details)"
  exit 1
fi
\rm -f log/*

#
# check tree construction
#
../src/${prog} -n 8 -f 1.0 -q -o log/know1.tree log/know1.gmm dat/know1.lfcc > log/${prog}.stdout 2> log/${prog}.stderr
diff log/know1.gmm  ref/init.2 > /dev/null 2> /dev/null
if test $? -eq 0 -a x`awk '{print NR}' log/${prog}.stderr | tail -1` = x; then
  diff log/know1.tree ref/init.tree > /dev/null 2> /dev/null
  if [ $? -eq 0 ]; then
    echo "  VQ test passed"
  else
    echo "  VQ test failed (see log/${prog}.stderr for details)"
    exit 1
  fi
else
  echo "  VQ test failed (see log/${prog}.stderr for details)"
  exit 1
fi
\rm -f log/*

#
# check script support 
#
../src/${prog} -n 8 -f 1.0 -L dat/script.lst log/know.gmm > log/${prog}.stdout 2> log/${prog}.stderr
diff log/know.gmm  ref/init.3 > /dev/null 2> /dev/null
if test $? -eq 0 -a x`awk '{print NR}' log/${prog}.stderr | tail -1` = x; then
  echo "  script test passed"
else
  echo "  script test failed (see log/${prog}.stderr for details)"
  exit 1
fi
\rm -f log/*

#
# check segment chopping 
#
../src/${prog} -n 8 -f 1.0 -L dat/script.lst -x spk1 log/know.gmm > log/${prog}.stdout 2> log/${prog}.stderr
diff log/know.gmm  ref/init.4 > /dev/null 2> /dev/null
if test $? -eq 0 -a x`awk '{print NR}' log/${prog}.stderr | tail -1` = x; then
  echo "  chopping test passed"
else
  echo "  chopping test failed (see log/${prog}.stderr for details)"
  exit 1
fi
\rm -f log/*

rmdir log